#
# Pipeline uni tests using BATS framework.
#

bats_require_minimum_version 1.5.0

setup() {
	# shellcheck source=./tests/tests.conf
	source "${PWD}/tests/tests.conf"
	make build >/dev/null
	DESTDIR="${temp}" make install >/dev/null
	install --no-target-directory \
		"${PWD}/tests/tests.conf" \
		"${temp}${sysconf_dir}/${name}/${name}.conf"
	# shellcheck source=./conf/errors.conf.sample
	source "${errs_file}"
}

teardown() {
	rm --recursive --force "${temp}"
	make clean >/dev/null
}

run_with_bad_arguments() { #@test
	run -1 "${pipeline_bin}"
	run -1 "${pipeline_bin}" bad_pipeline

	run -1 "${pipeline_bin}" newcommit
	run -1 "${pipeline_bin}" -H hash newcommit
	run -1 "${pipeline_bin}" -n name newcommit

	run -1 "${pipeline_bin}" newtag
	run -1 "${pipeline_bin}" -H hash newtag
	run -1 "${pipeline_bin}" -n name newtag
	run -1 "${pipeline_bin}" -T tag newtag
	run -1 "${pipeline_bin}" -H hash -n name newtag
	run -1 "${pipeline_bin}" -H hash -T tag newtag
	run -1 "${pipeline_bin}" -n name -T tag newtag

	run -1 "${pipeline_bin}" -c /path/to/nowhere -H hash -n name newcommit
	run -1 "${pipeline_bin}" -e /path/to/nowhere -H hash -n name newcommit
}

run_without_runner_sock() { #@test
	run "${pipeline_bin}" --conf="${conf_file}" --errs="${errs_file}" \
		--test --hash hash --name name newcommit
	echo "status: got ${status:?}, expected ${err_runner_sock_not_exist}"
	[ "${status}" -eq "${err_runner_sock_not_exist}" ]
}

run_newcommit_ok() { #@test
	ncat --listen --unixsock "${runner_sock}" &
	"${pipeline_bin}" --conf="${conf_file}" --errs="${errs_file}" \
		--test --hash hash --name name newcommit
	echo "done" | ncat --unixsock "${runner_sock}"
}

run_newtag_ok() { #@test
	ncat --listen --unixsock "${runner_sock}" &
	"${pipeline_bin}" --conf="${conf_file}" --errs="${errs_file}" \
		--test --hash hash --name name --tag tag newtag
	echo "done" | ncat --unixsock "${runner_sock}"
}
